@@ -4,7 +4,7 @@
  *
  *			  routines for working with Postgres's catalog and caches
  *
- * by Pavel Stehule 2013-2022
+ * by Pavel Stehule 2013-2023
  *
  *-------------------------------------------------------------------------
  */
@@ -12,6 +12,7 @@
 #include "plpgsql_check.h"
 
 #include "access/genam.h"
+
 #include "access/htup_details.h"
 
 #if PG_VERSION_NUM >= 120000
@@ -35,6 +36,8 @@
 #include "utils/syscache.h"
 #include "utils/regproc.h"
 
+#include "utils/rel.h"
+
 #if PG_VERSION_NUM >= 110000
 
 #include "catalog/pg_proc.h"
@@ -196,6 +199,8 @@ plpgsql_check_precheck_conditions(plpgsql_check_info *cinfo)
 	pfree(funcname);
 }
 
+#if PG_VERSION_NUM < 160000
+
 /*
  * plpgsql_check_get_extension_schema - given an extension OID, fetch its extnamespace
  *
@@ -256,6 +261,79 @@ get_extension_schema(Oid ext_oid)
 	return result;
 }
 
+#endif
+
+/*
+ * get_extension_version - given an extension OID, look up the version
+ *
+ * Returns a palloc'd string, or NULL if no such extension.
+ */
+char *
+get_extension_version(Oid ext_oid)
+{
+	char	   *result;
+	Relation	rel;
+	SysScanDesc scandesc;
+	HeapTuple	tuple;
+	ScanKeyData entry[1];
+
+#if PG_VERSION_NUM >= 120000
+
+	rel = table_open(ExtensionRelationId, AccessShareLock);
+
+	ScanKeyInit(&entry[0],
+				Anum_pg_extension_oid,
+				BTEqualStrategyNumber, F_OIDEQ,
+				ObjectIdGetDatum(ext_oid));
+
+#else
+
+	rel = heap_open(ExtensionRelationId, AccessShareLock);
+
+	ScanKeyInit(&entry[0],
+				ObjectIdAttributeNumber,
+				BTEqualStrategyNumber, F_OIDEQ,
+				ObjectIdGetDatum(ext_oid));
+
+#endif
+
+	scandesc = systable_beginscan(rel, ExtensionOidIndexId, true,
+								  NULL, 1, entry);
+
+	tuple = systable_getnext(scandesc);
+
+	/* We assume that there can be at most one matching tuple */
+	if (HeapTupleIsValid(tuple))
+	{
+		Datum		datum;
+		bool		isnull;
+
+		datum = heap_getattr(tuple, Anum_pg_extension_extversion,
+							 RelationGetDescr(rel), &isnull);
+
+		if (isnull)
+			elog(ERROR, "extversion is null");
+
+		result = text_to_cstring(DatumGetTextPP(datum));
+	}
+	else
+		result = NULL;
+
+	systable_endscan(scandesc);
+
+#if PG_VERSION_NUM >= 120000
+
+	table_close(rel, AccessShareLock);
+
+#else
+
+	heap_close(rel, AccessShareLock);
+
+#endif
+
+	return result;
+}
+
 /*
  * Returns oid of pragma function. It is used for elimination
  * pragma function from volatility tests.
