@@ -4,7 +4,7 @@
  *
  *			  pragma related code
  *
- * by Pavel Stehule 2013-2022
+ * by Pavel Stehule 2013-2023
  *
  *-------------------------------------------------------------------------
  */
@@ -217,6 +217,10 @@ pragma_apply(PLpgSQL_checkstate *cstate,
 	{
 		is_valid = plpgsql_check_pragma_table(cstate, pragma_str + 6, lineno);
 	}
+	else if (strncasecmp(pragma_str, "SEQUENCE:", 6) == 0)
+	{
+		is_valid = plpgsql_check_pragma_sequence(cstate, pragma_str + 9, lineno);
+	}
 	else
 	{
 		elog(WARNING, "unsupported pragma: %s", pragma_str);
